=begin
###
Copyright 2008-2021, TIG (c)
All rights reserved.
This software is provided "as is" and without any express or implied 
warranties, including, without limitation, the implied warranties of 
merchantability and fitness for a particular purpose.
###
Name:
default_layer_geometry.rb

Description:
Defaults the layer [tag] to Layer0 [Untagged] for all selected "geometry"
AND also any "geometry" inside selected Groups and Instances.
It iterates down into nested entities to process everything.
Only geometry is processed.
The layers set for any Groups and Instances themselves remain unchanged.

Usage:
1. Select one of more entities to be changed. 
2. Activate the script from the Plugins/Extensions Menu...

One step undoable.

Version:
1.0 20080310 First issue.
1.1 20120224 Updated.
2.0 20160108 Updated for speed, and signed for v2016. 
2.1 20160109 More optimization.  
3.0 20201211 Layers/Tags nomenclature added - re-signed.  
=end
###
module TIG
	module DefaultLayerGeometry 
    ###
    ### version check
    if Sketchup.version.to_i < 20
      @tlay="图层"
    else
      @tlay="标记"
    end
		###
		unless @loaded
			UI.menu("Plugins").add_item("Default #{@tlay} Geometry"){ self.new() }
			@loaded=true
			
			path = Sketchup.find_support_file("Plugins/default_layer_geometry")
			defaultlayergeometry_tb = UI::Toolbar.new("默认图层转换")
			cmd1 = UI::Command.new("默认图层转换") { self.new() }
			cmd1.small_icon = File.join(path,"icons/qctc024.png")
			cmd1.large_icon = File.join(path,"icons/qctc036.png")
			cmd1.tooltip = "默认图层转换"
			cmd1.status_bar_text = "将所选中的物体对象转到 layer0/未标记，如果选中的是组或组件会将最深层的未打组对象转换至layer0/未标记。"
			defaultlayergeometry_tb.add_item(cmd1)
			defaultlayergeometry_tb.show
		end#if
    ###
		def self.new()
			@model = Sketchup.active_model
			@layers = @model.layers
			ss = @model.selection
			@ecounter = 0
			@fcounter = 0
			@defs = []
			if ss[0]
				@model.start_operation("Default #{@tlay} Geometry", true)
					self.zero(ss)
				@model.commit_operation
				UI.messagebox("默认 #{@tlay} 转换:\n处理选择:\n#{@ecounter} 线\n#{@fcounter} 面")
			else
				UI.beep
				yn = UI.messagebox("默认 #{@tlay} 转换 ?\n是 \t= \t加工整体模型\n否 \t= \t退出，然后进行选择并重新运行", MB_YESNO)
				if yn == 6
					es = @model.entities.grep(Sketchup::Edge) + @model.entities.grep(Sketchup::Face)
					ds = @model.definitions.select{|e| ! e.image? }
					@model.start_operation("Default #{@tlay} Geometry", true)
						self.zero(es)
						self.zero(ds)
					@model.commit_operation
					UI.messagebox("默认 #{@tlay} 转换:\n加工整体模型:\n#{@ecounter} 线\n#{@fcounter} 面")
				else
					return nil
				end
			end#if
		end#def 
		###
		def self.zero( ents=[] )
			ents.each{|e|
				if e.is_a?(Sketchup::Edge)
					unless e.layer == @layers[0] ### Layer0/Untagged
						e.layer = nil
						@ecounter += 1
					end
				elsif e.is_a?(Sketchup::Face)
					unless e.layer == @layers[0] ### Layer0/Untagged
						e.layer = nil
						@fcounter += 1
					end
				elsif e.is_a?(Sketchup::ComponentInstance)
					defn = e.definition
					next if @defs.include?(defn)
					@defs << defn
					self.zero( defn.entities )
				elsif e.is_a?(Sketchup::Group)
					next if @defs.include?(e)
					@defs << e
					self.zero( e.entities )
				elsif e.is_a?(Sketchup::ComponentDefinition) && ! e.image?
					self.zero( e.entities )
				else
					### leave it alone
				end
			}
		end#def
		###
	end#mod
end#mod
